@@ -4,13 +4,14 @@ use 5.006;
 use Mojo::Base 'Mojolicious::Plugin';
 
 use HTML::Template::Pro;
+use HTML::Template::Pro::Extension;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 sub register {
 	my ( $self, $app, $conf ) = @_;
 	$self->{plugin_config} = $conf;
-	$app->renderer->add_handler( tmpl => sub{ $self->render_tmpl(@_)} );
+	$app->renderer->add_handler( tmpl => sub { $self->render_tmpl(@_) } );
 }
 
 sub render_tmpl {
@@ -32,43 +33,69 @@ sub render_tmpl {
 		push @template_dirs, $c->app->home->rel_dir("templates/$controller");
 	}
 
+	my $t;
 	my %t_options;
 
-	$t_options{die_on_bad_params}      = 0;
-	$t_options{global_vars}            = 1;
-	$t_options{loop_context_vars}      = 1;
-	$t_options{path}                   = \@template_dirs;
-	$t_options{search_path_on_include} = 1;
-
-	if ( defined( $options->{inline} ) ) {
-		$t_options{scalarref} = \$options->{inline};
-	}
-	elsif ( defined( $options->{template} ) ) {
-		if (defined (my $path = $r->template_path($options))) {
-			$t_options{filename} = $path;
-			$t_options{cache}    = 1;
-		} else {
-			$t_options{scalarref} = $r->get_data_template($options);
+	if ( $conf->{tmpl_opts}->{use_extension}
+		|| delete $tmpl_params{use_extension} )
+	{
+		if ( defined( $options->{inline} ) ) {
+			$t_options{tmplfile} = \$options->{inline};
+		}
+		elsif ( defined( $options->{template} ) ) {
+			if ( defined( my $path = $r->template_path($options) ) ) {
+				$t_options{tmplfile} = $path;
+				$t_options{cache}    = 1;
+			}
+			else {
+				$t_options{tmplfile} = \$r->get_data_template($options);
+			}
 		}
 
+		my $plugins = $conf->{tmpl_opts}->{plugins} || [];
+		$plugins = [ @$plugins, @{ $tmpl_params{plugins} } ]
+		  if exists $tmpl_params{plugins};
+		$t_options{plugins} = $plugins;
+		$t = new HTML::Template::Pro::Extension(%t_options);
 	}
+	else {
+		$t_options{die_on_bad_params}      = 0;
+		$t_options{global_vars}            = 1;
+		$t_options{loop_context_vars}      = 1;
+		$t_options{path}                   = \@template_dirs;
+		$t_options{search_path_on_include} = 1;
+
+		if ( defined( $options->{inline} ) ) {
+			$t_options{scalarref} = \$options->{inline};
+		}
+		elsif ( defined( $options->{template} ) ) {
+			if ( defined( my $path = $r->template_path($options) ) ) {
+				$t_options{filename} = $path;
+				$t_options{cache}    = 1;
+			}
+			else {
+				$t_options{scalarref} = $r->get_data_template($options);
+			}
+
+		}
+
+		$t = HTML::Template::Pro->new(
+			%t_options,
+			%{ $conf->{tmpl_opts} || {} },
+			%{ delete $tmpl_params{tmpl_opts} || {} }
+		);
+
+	}
+	unless ($t) { $r->render_exception("ERROR: No template created"); }
 
 	# sanity params removing scalar inside arrayref
 	foreach ( keys %tmpl_params ) {
 		delete $tmpl_params{$_}
 		  if ( ( ref $tmpl_params{$_} eq 'ARRAY' )
 			&& $tmpl_params{$_} > 0
-			&& $tmpl_params{$_}->[0] ne 'HASH' );
+			&& ref($tmpl_params{$_}->[0]) ne 'HASH' );
 	}
 
-	my $t = HTML::Template::Pro->new(
-		%t_options,
-		%{ $conf->{tmpl_opts} || {} },
-		%{ delete $tmpl_params{tmpl_opts} || {} }
-	);
-
-	unless ($t) { $r->render_exception("ERROR: No template created"); }
-
 	$t->param(%tmpl_params);
 
 	$$output = $t->output();
@@ -87,7 +114,7 @@ Mojolicious::Plugin::HTMLTemplateProRenderer - Mojolicious Plugin
   $self->plugin('HTMLTemplateProRenderer');
 
 
-  # Mojolicious::Lite
+  # Mojolicious::Lite 
   plugin 'HTMLTemplateProRenderer';
 
   # Render HTML::Template::Pro handler and post 'utf8 => 1' option for next HTML::Template::Pro->new call
@@ -100,15 +127,40 @@ Mojolicious::Plugin::HTMLTemplateProRenderer - Mojolicious Plugin
   # Set default options for all HTML::Template::Pro->new calls
   plugin 'HTMLTemplateProRenderer', tmpl_opts => {blind_cache => 1, open_mode => '<:encoding(UTF-16)'};
 
+  # Set use of L<HTML::Template::Pro::Extension>
+  plugin 'HTMLTemplateProRenderer', tmpl_opts => {use_extension => 1};
+
+  # render inline L<HTML::Template::Pro::Extension> using SLASH_VAR extension
+  get '/slash_var' => sub {
+    my $self = shift;
+    $self->stash(foo => 'bar');
+    $self->render(inline => '<p><TMPL_VAR NAME="foo">this will be deleted</TMPL_VAR></p>',
+      handler => 'tmpl', plugins => ['SLASH_VAR']);
+  };
+
+  # render a loop
+  get '/' => sub {
+    my $self = shift;
+    my $test = [{row => 'First row'},{row => 'Second row'}];
+    $self->stash(loop => $test);
+    $self->render(inline => '<ul><TMPL_LOOP NAME="loop"><li><TMPL_VAR NAME="row"></li></TMPL_LOOP></ul>',
+      handler => 'tmpl');
+  };
+
 =head1 DESCRIPTION
 
-L<Mojolicious::Plugin::HTMLTemplateProRenderer> is a L<Mojolicious> plugin to use L<HTML::Template::Pro> module in your Mojo projects.
+L<Mojolicious::Plugin::HTMLTemplateProRenderer> is a L<Mojolicious> plugin
+to use L<HTML::Template::Pro> and L<HTML::Template::Pro::Extension>
+modules in your Mojo projects.
 
 L<HTML::Template::Pro> is a fast lightweight C/Perl+XS reimplementation of L<HTML::Template> (as of 2.9) and L<HTML::Template::Expr> (as of 0.0.7). 
 It is not intended to be a complete replacement, but to be a fast implementation of L<HTML::Template> if you don't need querying, the extended facility of L<HTML::Template>.
 
 Designed for heavy upload, resource limitations, abcence of L<mod_perl>.
 
+L<HTML::Template::Pro::Extension> is a pluggable extension syntax module
+for L<HTML::Template::Pro>.
+
 =head1 METHODS
 
 L<Mojolicious::Plugin::HTMLTemplateProRenderer> inherits all methods from
@@ -130,6 +182,12 @@ These are options for L<Mojolicious::Plugin::HTMLTemplateProRenderer>
 
 Templates are found starting from home base app path other than home_app/templates path.
 
+=head2 C<use_extension>
+
+  $self->render('template', handler => 'tmpl',use_extension => 1, plugins => ['SLASH_VAR']
+
+Enable use of L<HTML::Template::Pro::Extension> and use of plugins.
+
 =head1 SEE ALSO
 
 L<Mojolicious>, L<Mojolicious::Guides>, L<http://mojolicio.us>.
